import 'dart:async';
import 'package:rx_property/rx_property.dart';
import 'package:rxdart/rxdart.dart';
import 'package:test/test.dart';

void main() {
  ReactiveProperty reactiveCounter;

  setUp(() {
    reactiveCounter = ReactiveProperty(10);
  });

  group('state', () {
    test('shoud start with initial state', () {
      expect(reactiveCounter.value, 10);
    });

    test('shoud be broadcast', () {
      expect(reactiveCounter.isBroadcast, true);
    });

    test('shoud return the current state after add a new state', () {
      reactiveCounter.add(20);
      expect(reactiveCounter.value, 20);
    });
  });

  group('computed', () {
    test('shoud compute the lastes value', () async {
      final firstName = ReactiveProperty<String>('');
      final lastName = ReactiveProperty<String>('');
      firstName.value = "Pedro";
      await Future.delayed(Duration.zero);
      lastName.value = 'Bissonho';
      await Future.delayed(Duration.zero);
      lastName.value = 'XXX';
      await Future.delayed(Duration.zero);
      Stream<String> fullName;

      fullName = computed(firstName, lastName, (name, last) => "$name $last");

      expectLater(fullName, emits('Pedro XXX'));
      expect(firstName.value, 'Pedro');
      expect(lastName.value, 'XXX');
    });
  });

  test("description", () async {
    final rxList = ReplayReactiveProperty<List<int>>([]);
    rxList.change((value) => List.from(value)..add(10));
    await Future.delayed(Duration(milliseconds: 1));
    rxList.change((value) => List.from(value)..add(20));
    await Future.delayed(Duration(milliseconds: 1));
    rxList.change((value) => List.from(value)..add(30));

        await Future.delayed(Duration(milliseconds: 500));

    rxList.listen((value) {
      print(value);
    });

    await Future.delayed(Duration(milliseconds: 500));
  });

  group('emit new state', () {
    group('add', () {
      test('shoud emit corret state', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20]));

        reactiveCounter.add(20);
      });

      test('shoud emit corret state ir order', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20, 30]));

        reactiveCounter.add(20);
        await Future.delayed(Duration.zero);
        reactiveCounter.add(30);
      });
    });

    group('state set', () {
      test('shoud emit corret state', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20]));

        reactiveCounter.add(20);
      });

      test('shoud emit corret state ir order', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 20, 30]));

        reactiveCounter.value = 20;
        await Future.delayed(Duration.zero);
        reactiveCounter.value = 30;
        ;
      });

      test('get and set state', () async {
        expectLater(reactiveCounter, emitsInOrder([10, 11, 12]));

        reactiveCounter.value++;
        await Future.delayed(Duration.zero);
        reactiveCounter.value++;
        ;
      });
    });
  });

  group('listen', () {});

  group('close', () {});
}
